@@ -332,29 +332,29 @@ LIBYANG_API_DECL LY_ERR lyd_parse_ext_data(const struct lysc_ext_instance *ext,
  * @{
  */
 enum lyd_type {
-    LYD_TYPE_DATA_YANG = 0,     /* generic YANG instance data */
-    LYD_TYPE_RPC_YANG,          /* instance of a YANG RPC/action request with only "input" data children,
-                                   including all parents and optional top-level "action" element in case of an action */
-    LYD_TYPE_NOTIF_YANG,        /* instance of a YANG notification, including all parents in case of a nested one */
-    LYD_TYPE_REPLY_YANG,        /* instance of a YANG RPC/action reply with only "output" data children,
-                                   including all parents in case of an action */
+    LYD_TYPE_DATA_YANG = 0,     /**< generic YANG instance data */
+    LYD_TYPE_RPC_YANG,          /**< instance of a YANG RPC/action request with only "input" data children,
+                                     including all parents and optional top-level "action" element in case of an action */
+    LYD_TYPE_NOTIF_YANG,        /**< instance of a YANG notification, including all parents in case of a nested one */
+    LYD_TYPE_REPLY_YANG,        /**< instance of a YANG RPC/action reply with only "output" data children,
+                                     including all parents in case of an action */
 
-    LYD_TYPE_RPC_NETCONF,       /* complete NETCONF RPC invocation as defined for
-                                   [RPC](https://tools.ietf.org/html/rfc7950#section-7.14.4) and
-                                   [action](https://tools.ietf.org/html/rfc7950#section-7.15.2) */
-    LYD_TYPE_NOTIF_NETCONF,     /* complete NETCONF notification message as defined for
-                                   [notification](https://tools.ietf.org/html/rfc7950#section-7.16.2) */
-    LYD_TYPE_REPLY_NETCONF,     /* complete NETCONF RPC reply as defined for
-                                   [RPC](https://tools.ietf.org/html/rfc7950#section-7.14.4) and
-                                   [action](https://tools.ietf.org/html/rfc7950#section-7.15.2) */
+    LYD_TYPE_RPC_NETCONF,       /**< complete NETCONF RPC invocation as defined for
+                                     [RPC](https://tools.ietf.org/html/rfc7950#section-7.14.4) and
+                                     [action](https://tools.ietf.org/html/rfc7950#section-7.15.2) */
+    LYD_TYPE_NOTIF_NETCONF,     /**< complete NETCONF notification message as defined for
+                                     [notification](https://tools.ietf.org/html/rfc7950#section-7.16.2) */
+    LYD_TYPE_REPLY_NETCONF,     /**< complete NETCONF RPC reply as defined for
+                                     [RPC](https://tools.ietf.org/html/rfc7950#section-7.14.4) and
+                                     [action](https://tools.ietf.org/html/rfc7950#section-7.15.2) */
 
-    LYD_TYPE_RPC_RESTCONF,      /* message-body of a RESTCONF operation input parameters
-                                   ([ref](https://www.rfc-editor.org/rfc/rfc8040.html#section-3.6.1)) */
-    LYD_TYPE_NOTIF_RESTCONF,    /* RESTCONF JSON notification data
-                                   ([ref](https://www.rfc-editor.org/rfc/rfc8040.html#section-6.4)), to parse
-                                   a notification in XML, use ::LYD_TYPE_NOTIF_NETCONF */
-    LYD_TYPE_REPLY_RESTCONF     /* message-body of a RESTCONF operation output parameters
-                                   ([ref](https://www.rfc-editor.org/rfc/rfc8040.html#section-3.6.2)) */
+    LYD_TYPE_RPC_RESTCONF,      /**< message-body of a RESTCONF operation input parameters
+                                     ([ref](https://www.rfc-editor.org/rfc/rfc8040.html#section-3.6.1)) */
+    LYD_TYPE_NOTIF_RESTCONF,    /**< RESTCONF JSON notification data
+                                     ([ref](https://www.rfc-editor.org/rfc/rfc8040.html#section-6.4)), to parse
+                                     a notification in XML, use ::LYD_TYPE_NOTIF_NETCONF */
+    LYD_TYPE_REPLY_RESTCONF     /**< message-body of a RESTCONF operation output parameters
+                                     ([ref](https://www.rfc-editor.org/rfc/rfc8040.html#section-3.6.2)) */
 };
 /** @} datatype */
 
@@ -409,13 +409,21 @@ enum lyd_type {
  *   - @p op - must be NULL, @p parent points to the operation;
  *   Note that error reply should be parsed as 'yang-data' extension data.
  *
+ * - ::LYD_TYPE_RPC_YANG:
+ * - ::LYD_TYPE_NOTIF_YANG:
+ * - ::LYD_TYPE_REPLY_YANG:
+ *   - all the parameters have their default meaning.
+ *
+ * - :: LYD_TYPE_DATA_YANG:
+ *   - not accepted by the function.
+ *
  * @param[in] ctx libyang context.
  * @param[in] parent Optional parent to connect the parsed nodes to.
  * @param[in] in Input handle to read the input from.
  * @param[in] format Expected format of the data in @p in.
  * @param[in] data_type Expected operation to parse (@ref datatype).
  * @param[out] tree Optional full parsed data tree. If @p parent is set, set to NULL.
- * @param[out] op Optional pointer to the operation (action/RPC) node.
+ * @param[out] op Optional pointer to the operation (action/RPC/notification) node.
  * @return LY_ERR value.
  * @return LY_ENOT if @p data_type is a NETCONF message and the root XML element is not the expected one.
  */
