@@ -53,6 +53,30 @@ lyd_xml_ctx_free(struct lyd_ctx *lydctx)
     free(ctx);
 }
 
+/**
+ * @brief Log namespace error.
+ *
+ * @param[in] xmlctx XML context
+ * @param[in] prefix XML prefix.
+ * @param[in] prefix_len XML prefix length.
+ * @param[in] attr_name Current XML attribute name.
+ * @param[in] attr_len Current XML attribute name length.
+ */
+static void
+lydxml_log_namespace_err(struct lyxml_ctx *xmlctx, const char *prefix, size_t prefix_len,
+        const char *attr_name, size_t attr_len)
+{
+    if (prefix_len && attr_len) {
+        LOGVAL(xmlctx->ctx, LYVE_REFERENCE, "Unknown XML prefix \"%.*s\" at attribute \"%.*s\".",
+                (int)prefix_len, prefix, (int)attr_len, attr_name);
+    } else if (prefix_len) {
+        LOGVAL(xmlctx->ctx, LYVE_REFERENCE, "Unknown XML prefix \"%.*s\".",
+                (int)prefix_len, prefix);
+    } else {
+        LOGVAL(xmlctx->ctx, LYVE_REFERENCE, "Missing XML namespace.");
+    }
+}
+
 /**
  * @brief Parse and create XML metadata.
  *
@@ -109,8 +133,10 @@ lydxml_metadata(struct lyd_xml_ctx *lydctx, const struct lysc_node *sparent, str
             if (lydctx->parse_opts & LYD_PARSE_STRICT) {
                 LOGVAL(xmlctx->ctx, LYVE_REFERENCE, "Missing mandatory prefix for XML metadata \"%.*s\".",
                         (int)xmlctx->name_len, xmlctx->name);
-                ret = LY_EVALID;
-                goto cleanup;
+                /* If LYD_VALIDATE_MULTI_ERROR is set, then continue parsing, because otherwise the parser context
+                 * will remain in a bad state, which will cause termination on some assert or undefined behavior.
+                 */
+                LY_DPARSER_ERR_GOTO(LY_EVALID, ret = LY_EVALID, lydctx, cleanup);
             }
 
             /* skip attr */
@@ -123,8 +149,7 @@ lydxml_metadata(struct lyd_xml_ctx *lydctx, const struct lysc_node *sparent, str
         /* get namespace of the attribute to find its annotation definition */
         ns = lyxml_ns_get(&xmlctx->ns, xmlctx->prefix, xmlctx->prefix_len);
         if (!ns) {
-            /* unknown namespace, XML error */
-            LOGVAL(xmlctx->ctx, LYVE_REFERENCE, "Unknown XML prefix \"%.*s\".", (int)xmlctx->prefix_len, xmlctx->prefix);
+            lydxml_log_namespace_err(xmlctx, xmlctx->prefix, xmlctx->prefix_len, xmlctx->name, xmlctx->name_len);
             ret = LY_ENOTFOUND;
             goto cleanup;
         }
@@ -215,7 +240,7 @@ lydxml_attrs(struct lyxml_ctx *xmlctx, struct lyd_attr **attr)
         if (prefix_len) {
             ns = lyxml_ns_get(&xmlctx->ns, prefix, prefix_len);
             if (!ns) {
-                LOGVAL(xmlctx->ctx, LYVE_REFERENCE, "Unknown XML prefix \"%.*s\".", (int)prefix_len, prefix);
+                lydxml_log_namespace_err(xmlctx, prefix, prefix_len, name, name_len);
                 ret = LY_EVALID;
                 goto cleanup;
             }
@@ -369,8 +394,9 @@ lydxml_data_skip(struct lyxml_ctx *xmlctx)
 static LY_ERR
 lydxml_data_check_opaq(struct lyd_xml_ctx *lydctx, const struct lysc_node **snode)
 {
-    LY_ERR ret = LY_SUCCESS;
+    LY_ERR ret = LY_SUCCESS, r;
     struct lyxml_ctx *xmlctx = lydctx->xmlctx, pxmlctx;
+    uint32_t *prev_lo, temp_lo = 0;
 
     if (!(lydctx->parse_opts & LYD_PARSE_OPAQ)) {
         /* only checks specific to opaque nodes */
@@ -395,7 +421,10 @@ lydxml_data_check_opaq(struct lyd_xml_ctx *lydctx, const struct lysc_node **snod
 
     if ((*snode)->nodetype & LYD_NODE_TERM) {
         /* value may not be valid in which case we parse it as an opaque node */
-        if (ly_value_validate(NULL, *snode, xmlctx->value, xmlctx->value_len, LY_VALUE_XML, &xmlctx->ns, LYD_HINT_DATA)) {
+        prev_lo = ly_temp_log_options(&temp_lo);
+        r = ly_value_validate(NULL, *snode, xmlctx->value, xmlctx->value_len, LY_VALUE_XML, &xmlctx->ns, LYD_HINT_DATA);
+        ly_temp_log_options(prev_lo);
+        if (r) {
             LOGVRB("Parsing opaque term node \"%s\" with invalid value \"%.*s\".", (*snode)->name, (int)xmlctx->value_len,
                     xmlctx->value);
             *snode = NULL;
@@ -441,7 +470,9 @@ lydxml_get_hints_opaq(const char *name, size_t name_len, const char *value, size
 {
     struct lyd_node_opaq *opaq;
     char *ptr;
-    long num;
+    /* this needs to be at least 64bit, and it "should not" be an explicit int64_t
+     * because the code calls strtoll later on, which "might" return a bigger type */
+    long long num;
 
     *hints = 0;
     *anchor = NULL;
@@ -453,11 +484,11 @@ lydxml_get_hints_opaq(const char *name, size_t name_len, const char *value, size
         /* boolean value */
         *hints |= LYD_VALHINT_BOOLEAN;
     } else {
-        num = strtol(value, &ptr, 10);
+        num = strtoll(value, &ptr, 10);
         if ((unsigned)(ptr - value) == value_len) {
             /* number value */
             *hints |= LYD_VALHINT_DECNUM;
-            if ((num < INT32_MIN) || (num > (long)UINT32_MAX)) {
+            if ((num < INT32_MIN) || (num > UINT32_MAX)) {
                 /* large number */
                 *hints |= LYD_VALHINT_NUM64;
             }
@@ -535,12 +566,7 @@ lydxml_subtree_get_snode(struct lyd_xml_ctx *lydctx, const struct lyd_node *pare
         if (lydctx->int_opts & LYD_INTOPT_ANY) {
             goto unknown_module;
         }
-
-        if (prefix_len) {
-            LOGVAL(ctx, LYVE_REFERENCE, "Unknown XML prefix \"%.*s\".", (int)prefix_len, prefix);
-        } else {
-            LOGVAL(ctx, LYVE_REFERENCE, "Missing XML namespace.");
-        }
+        lydxml_log_namespace_err(xmlctx, prefix, prefix_len, NULL, 0);
         return LY_EVALID;
     }
 
@@ -841,19 +867,14 @@ lydxml_subtree_inner(struct lyd_xml_ctx *lydctx, const struct lysc_node *snode,
 
     if (snode->nodetype == LYS_LIST) {
         /* check all keys exist */
-        r = lyd_parse_check_keys(*node);
+        r = lyd_parser_check_keys(*node);
         LY_CHECK_ERR_GOTO(r, rc = r, cleanup);
     }
 
     if (!(lydctx->parse_opts & LYD_PARSE_ONLY) && !rc) {
-        /* new node validation, autodelete CANNOT occur (it can if multi-error), all nodes are new */
-        r = lyd_validate_new(lyd_node_child_p(*node), snode, NULL, lydctx->val_opts, NULL);
+        /* new node validation */
+        r = lyd_parser_validate_new_implicit((struct lyd_ctx *)lydctx, *node);
         LY_DPARSER_ERR_GOTO(r, rc = r, lydctx, cleanup);
-
-        /* add any missing default children */
-        r = lyd_new_implicit_r(*node, lyd_node_child_p(*node), NULL, NULL, &lydctx->node_when, &lydctx->node_types,
-                &lydctx->ext_node, (lydctx->val_opts & LYD_VALIDATE_NO_STATE) ? LYD_IMPLICIT_NO_STATE : 0, NULL);
-        LY_CHECK_ERR_GOTO(r, rc = r, cleanup);
     }
 
     if (snode->nodetype & (LYS_RPC | LYS_ACTION | LYS_NOTIF)) {
@@ -1056,14 +1077,13 @@ lydxml_subtree_r(struct lyd_xml_ctx *lydctx, struct lyd_node *parent, struct lyd
     /* create metadata/attributes */
     if (xmlctx->status == LYXML_ATTRIBUTE) {
         if (snode) {
-            rc = lydxml_metadata(lydctx, snode, &meta);
-            LY_CHECK_GOTO(rc, cleanup);
+            r = lydxml_metadata(lydctx, snode, &meta);
         } else {
             assert(lydctx->parse_opts & LYD_PARSE_OPAQ);
-            rc = lydxml_attrs(xmlctx, &attr);
-            LY_CHECK_GOTO(rc, cleanup);
+            r = lydxml_attrs(xmlctx, &attr);
         }
     }
+    LY_DPARSER_ERR_GOTO(r, rc = r, lydctx, cleanup);
 
     assert(xmlctx->status == LYXML_ELEM_CONTENT);
     if (!snode) {
@@ -1086,7 +1106,7 @@ lydxml_subtree_r(struct lyd_xml_ctx *lydctx, struct lyd_node *parent, struct lyd
 node_parsed:
     if (node && snode) {
         /* add/correct flags */
-        r = lyd_parse_set_data_flags(node, &meta, (struct lyd_ctx *)lydctx, ext);
+        r = lyd_parser_set_data_flags(node, &meta, (struct lyd_ctx *)lydctx, ext);
         LY_CHECK_ERR_GOTO(r, rc = r; lyd_free_tree(node), cleanup);
 
         if (!(lydctx->parse_opts & LYD_PARSE_ONLY)) {
@@ -1175,7 +1195,7 @@ lydxml_envelope(struct lyxml_ctx *xmlctx, const char *name, const char *uri, ly_
     prefix_len = xmlctx->prefix_len;
     ns = lyxml_ns_get(&xmlctx->ns, prefix, prefix_len);
     if (!ns) {
-        LOGVAL(xmlctx->ctx, LYVE_REFERENCE, "Unknown XML prefix \"%.*s\".", (int)prefix_len, prefix);
+        lydxml_log_namespace_err(xmlctx, prefix, prefix_len, NULL, 0);
         return LY_EVALID;
     } else if (strcmp(ns->uri, uri)) {
         /* different namespace */
@@ -1399,7 +1419,7 @@ lydxml_opaq_r(struct lyxml_ctx *xmlctx, struct lyd_node *parent)
     prefix_len = xmlctx->prefix_len;
     ns = lyxml_ns_get(&xmlctx->ns, prefix, prefix_len);
     if (!ns) {
-        LOGVAL(xmlctx->ctx, LYVE_REFERENCE, "Unknown XML prefix \"%.*s\".", (int)prefix_len, prefix);
+        lydxml_log_namespace_err(xmlctx, prefix, prefix_len, NULL, 0);
         return LY_EVALID;
     }
 
