@@ -4,7 +4,7 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief Built-in types plugins and interface for user types plugins.
  *
- * Copyright (c) 2019 - 2022 CESNET, z.s.p.o.
+ * Copyright (c) 2019 - 2024 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -352,10 +352,16 @@ lyplg_type_print_simple(const struct ly_ctx *UNUSED(ctx), const struct lyd_value
 LIBYANG_API_DEF LY_ERR
 lyplg_type_dup_simple(const struct ly_ctx *ctx, const struct lyd_value *original, struct lyd_value *dup)
 {
-    memset(dup, 0, sizeof *dup);
-    LY_CHECK_RET(lydict_insert(ctx, original->_canonical, 0, &dup->_canonical));
+    LY_ERR r;
+
+    if ((r = lydict_dup(ctx, original->_canonical, &dup->_canonical))) {
+        /* in case of error NULL the values so that freeing does not fail */
+        memset(dup, 0, sizeof *dup);
+        return r;
+    }
     memcpy(dup->fixed_mem, original->fixed_mem, sizeof dup->fixed_mem);
     dup->realtype = original->realtype;
+
     return LY_SUCCESS;
 }
 
@@ -520,37 +526,18 @@ decimal:
 LIBYANG_API_DEF LY_ERR
 lyplg_type_validate_patterns(struct lysc_pattern **patterns, const char *str, size_t str_len, struct ly_err_item **err)
 {
-    int rc, match_opts;
+    LY_ERR r;
     LY_ARRAY_COUNT_TYPE u;
-    pcre2_match_data *match_data = NULL;
 
     LY_CHECK_ARG_RET(NULL, str, err, LY_EINVAL);
 
     *err = NULL;
 
     LY_ARRAY_FOR(patterns, u) {
-        /* match_data needs to be allocated each time because of possible multi-threaded evaluation */
-        match_data = pcre2_match_data_create_from_pattern(patterns[u]->code, NULL);
-        if (!match_data) {
-            return ly_err_new(err, LY_EMEM, 0, NULL, NULL, LY_EMEM_MSG);
-        }
+        r = ly_pattern_code_match(patterns[u]->code, str, str_len, err);
+        LY_CHECK_RET(r && (r != LY_ENOT), r);
 
-        match_opts = PCRE2_ANCHORED;
-#ifdef PCRE2_ENDANCHORED
-        /* PCRE2_ENDANCHORED was added in PCRE2 version 10.30 */
-        match_opts |= PCRE2_ENDANCHORED;
-#endif
-        rc = pcre2_match(patterns[u]->code, (PCRE2_SPTR)str, str_len, 0, match_opts, match_data, NULL);
-        pcre2_match_data_free(match_data);
-
-        if ((rc != PCRE2_ERROR_NOMATCH) && (rc < 0)) {
-            PCRE2_UCHAR pcre2_errmsg[LY_PCRE2_MSG_LIMIT] = {0};
-
-            pcre2_get_error_message(rc, pcre2_errmsg, LY_PCRE2_MSG_LIMIT);
-
-            return ly_err_new(err, LY_ESYS, 0, NULL, NULL, "%s", (const char *)pcre2_errmsg);
-        } else if (((rc == PCRE2_ERROR_NOMATCH) && !patterns[u]->inverted) ||
-                ((rc != PCRE2_ERROR_NOMATCH) && patterns[u]->inverted)) {
+        if (((r == LY_ENOT) && !patterns[u]->inverted) || ((r == LY_SUCCESS) && patterns[u]->inverted)) {
             char *eapptag = patterns[u]->eapptag ? strdup(patterns[u]->eapptag) : NULL;
 
             if (patterns[u]->emsg) {
@@ -563,6 +550,7 @@ lyplg_type_validate_patterns(struct lysc_pattern **patterns, const char *str, si
             }
         }
     }
+
     return LY_SUCCESS;
 }
 
@@ -831,8 +819,9 @@ lyplg_type_lypath_new(const struct ly_ctx *ctx, const char *value, size_t value_
     uint32_t prefix_opt = 0;
     struct ly_err_item *e;
     const char *err_fmt = NULL;
+    uint16_t oper;
 
-    LY_CHECK_ARG_RET(ctx, ctx, value, ctx_node, path, err, LY_EINVAL);
+    LY_CHECK_ARG_RET(ctx, ctx, value, path, err, LY_EINVAL);
 
     *path = NULL;
     *err = NULL;
@@ -871,8 +860,8 @@ lyplg_type_lypath_new(const struct ly_ctx *ctx, const char *value, size_t value_
     }
 
     /* resolve it on schema tree */
-    ret = ly_path_compile(ctx, NULL, ctx_node, NULL, exp, (ctx_node->flags & LYS_IS_OUTPUT) ?
-            LY_PATH_OPER_OUTPUT : LY_PATH_OPER_INPUT, LY_PATH_TARGET_SINGLE, 1, format, prefix_data, path);
+    oper = (ctx_node && (ctx_node->flags & LYS_IS_OUTPUT)) ? LY_PATH_OPER_OUTPUT : LY_PATH_OPER_INPUT;
+    ret = ly_path_compile(ctx, NULL, ctx_node, NULL, exp, oper, LY_PATH_TARGET_SINGLE, 1, format, prefix_data, path);
     if (ret) {
         err_fmt = "Invalid instance-identifier \"%.*s\" value - semantic error%s%s";
         goto cleanup;
@@ -888,7 +877,7 @@ cleanup:
             ly_err_clean((struct ly_ctx *)ctx, e);
         }
 
-        ly_path_free(ctx, *path);
+        ly_path_free(*path);
         *path = NULL;
     }
 
@@ -896,9 +885,9 @@ cleanup:
 }
 
 LIBYANG_API_DEF void
-lyplg_type_lypath_free(const struct ly_ctx *ctx, struct ly_path *path)
+lyplg_type_lypath_free(const struct ly_ctx *UNUSED(ctx), struct ly_path *path)
 {
-    ly_path_free(ctx, path);
+    ly_path_free(path);
 }
 
 LIBYANG_API_DEF LY_ERR
@@ -1007,7 +996,7 @@ lyplg_type_resolve_leafref_get_target_path(const struct lyxp_expr *path, const s
     LY_CHECK_GOTO(lyxp_expr_parse(ctx_node->module->ctx, str_path, 0, 1, target_path), cleanup);
 
 cleanup:
-    ly_path_free(ctx_node->module->ctx, p);
+    ly_path_free(p);
     free(str_path);
     return rc;
 }
