@@ -18,10 +18,10 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "common.h"
 #include "context.h"
 #include "dict.h"
 #include "log.h"
+#include "ly_common.h"
 #include "out.h"
 #include "out_internal.h"
 #include "parser_data.h"
@@ -209,6 +209,10 @@ xml_print_meta(struct xmlpr_ctx *pctx, const struct lyd_node *node)
     }
 
     for (meta = node->meta; meta; meta = meta->next) {
+        if (!lyd_metadata_should_print(meta)) {
+            continue;
+        }
+
         /* store the module of the default namespace, NULL because there is none */
         ly_set_add(&ns_list, NULL, 0, NULL);
 
@@ -410,7 +414,7 @@ xml_print_anydata(struct xmlpr_ctx *pctx, const struct lyd_node_any *node)
 {
     struct lyd_node_any *any = (struct lyd_node_any *)node;
     struct lyd_node *iter;
-    uint32_t prev_opts, temp_lo = 0;
+    uint32_t prev_opts, *prev_lo, temp_lo = 0;
     LY_ERR ret;
 
     if ((node->schema->nodetype == LYS_ANYDATA) && (node->value_type != LYD_ANYDATA_DATATREE)) {
@@ -427,7 +431,7 @@ no_content:
     } else {
         if (any->value_type == LYD_ANYDATA_LYB) {
             /* turn logging off */
-            ly_temp_log_options(&temp_lo);
+            prev_lo = ly_temp_log_options(&temp_lo);
 
             /* try to parse it into a data tree */
             if (lyd_parse_data_mem((struct ly_ctx *)LYD_CTX(node), any->value.mem, LYD_LYB,
@@ -439,7 +443,7 @@ no_content:
             }
 
             /* turn logging on again */
-            ly_temp_log_options(NULL);
+            ly_temp_log_options(prev_lo);
         }
 
         switch (any->value_type) {
