@@ -4,7 +4,7 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief Parsing and validation common functions for data trees
  *
- * Copyright (c) 2015 - 2022 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2024 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -33,6 +33,7 @@
 #include "parser_data.h"
 #include "plugins_exts.h"
 #include "printer_data.h"
+#include "schema_compile_node.h"
 #include "set.h"
 #include "tree.h"
 #include "tree_data.h"
@@ -1157,35 +1158,6 @@ lyd_meta_is_internal(const struct lyd_meta *meta)
     return 0;
 }
 
-void
-lyd_cont_set_dflt(struct lyd_node *node)
-{
-    const struct lyd_node *child;
-
-    while (node) {
-        if (!node->schema || (node->flags & LYD_DEFAULT) || !lysc_is_np_cont(node->schema)) {
-            /* not a non-dflt NP container */
-            break;
-        }
-
-        LY_LIST_FOR(lyd_child(node), child) {
-            if (!(child->flags & LYD_DEFAULT)) {
-                break;
-            }
-        }
-        if (child) {
-            /* explicit child, no dflt change */
-            break;
-        }
-
-        /* set the dflt flag */
-        node->flags |= LYD_DEFAULT;
-
-        /* check all parent containers */
-        node = lyd_parent(node);
-    }
-}
-
 /**
  * @brief Comparison callback to match schema node with a schema of a data node.
  *
@@ -1301,6 +1273,44 @@ lyd_del_move_root(struct lyd_node **root, const struct lyd_node *to_del, const s
     }
 }
 
+void
+lyd_np_cont_dflt_set(struct lyd_node *parent)
+{
+    const struct lyd_node *child;
+
+    while (parent) {
+        if (!parent->schema || (parent->flags & LYD_DEFAULT) || !lysc_is_np_cont(parent->schema)) {
+            /* not a non-dflt NP container */
+            break;
+        }
+
+        LY_LIST_FOR(lyd_child(parent), child) {
+            if (!(child->flags & LYD_DEFAULT)) {
+                break;
+            }
+        }
+        if (child) {
+            /* explicit child, no dflt change */
+            break;
+        }
+
+        /* set the dflt flag */
+        parent->flags |= LYD_DEFAULT;
+
+        /* check all parent containers */
+        parent = lyd_parent(parent);
+    }
+}
+
+void
+lyd_np_cont_dflt_del(struct lyd_node *parent)
+{
+    while (parent && (parent->flags & LYD_DEFAULT)) {
+        parent->flags &= ~LYD_DEFAULT;
+        parent = lyd_parent(parent);
+    }
+}
+
 LY_ERR
 ly_nested_ext_schema(const struct lyd_node *parent, const struct lysc_node *sparent, const char *prefix,
         size_t prefix_len, LY_VALUE_FORMAT format, void *prefix_data, const char *name, size_t name_len,
@@ -1834,3 +1844,80 @@ ly_time_ts2str(const struct timespec *ts, char **str)
 
     return ly_time_time2str(ts->tv_sec, ts->tv_nsec ? frac_buf : NULL, str);
 }
+
+LY_ERR
+ly_pattern_code_match(pcre2_code *pcode, const char *str, size_t str_len, struct ly_err_item **err)
+{
+    int r, match_opts;
+    pcre2_match_data *match_data = NULL;
+
+    /* match_data needs to be allocated each time because of possible multi-threaded evaluation */
+    match_data = pcre2_match_data_create_from_pattern(pcode, NULL);
+    if (!match_data) {
+        return ly_err_new(err, LY_EMEM, 0, NULL, NULL, LY_EMEM_MSG);
+    }
+
+    match_opts = PCRE2_ANCHORED;
+#ifdef PCRE2_ENDANCHORED
+    /* PCRE2_ENDANCHORED was added in PCRE2 version 10.30 */
+    match_opts |= PCRE2_ENDANCHORED;
+#endif
+    r = pcre2_match(pcode, (PCRE2_SPTR)str, str_len, 0, match_opts, match_data, NULL);
+    pcre2_match_data_free(match_data);
+
+    if ((r != PCRE2_ERROR_NOMATCH) && (r < 0)) {
+        PCRE2_UCHAR pcre2_errmsg[LY_PCRE2_MSG_LIMIT] = {0};
+
+        pcre2_get_error_message(r, pcre2_errmsg, LY_PCRE2_MSG_LIMIT);
+        return ly_err_new(err, LY_ESYS, 0, NULL, NULL, "%s", (const char *)pcre2_errmsg);
+    }
+
+    return (r == PCRE2_ERROR_NOMATCH) ? LY_ENOT : LY_SUCCESS;
+}
+
+LIBYANG_API_DEF LY_ERR
+ly_pattern_match(const struct ly_ctx *ctx, const char *pattern, const char *string, uint32_t str_len, pcre2_code **pcode)
+{
+    LY_ERR r;
+    pcre2_code *code;
+    struct ly_err_item *err = NULL;
+
+    LY_CHECK_ARG_RET(ctx, pattern || (pcode && *pcode), string, LY_EINVAL);
+
+    if (pcode && *pcode) {
+        /* use precompiled pattern */
+        code = *pcode;
+    } else {
+        /* compile the pattern */
+        LY_CHECK_RET(lys_compile_type_pattern_check(ctx, pattern, &code));
+    }
+
+    /* match */
+    if (!str_len) {
+        str_len = strlen(string);
+    }
+    r = ly_pattern_code_match(code, string, str_len, &err);
+    if (r && (r != LY_ENOT)) {
+        ly_err_print(ctx, err);
+        ly_err_free(err);
+        return r;
+    }
+
+    if (pcode && !*pcode) {
+        *pcode = code;
+    } else if (!pcode || !*pcode) {
+        pcre2_code_free(code);
+    }
+    return r;
+}
+
+LIBYANG_API_DEF LY_ERR
+ly_pattern_compile(const struct ly_ctx *ctx, const char *pattern, pcre2_code **pcode)
+{
+    LY_CHECK_ARG_RET(ctx, pattern, pcode, LY_EINVAL);
+
+    *pcode = NULL;
+
+    /* compile the pattern */
+    return lys_compile_type_pattern_check(ctx, pattern, pcode);
+}
