@@ -91,8 +91,8 @@ lysp_check_date(struct lysp_ctx *ctx, const char *date, size_t date_len, const c
     }
     memcpy(&tm_, &tm, sizeof tm);
 
-    /* DST may move the hour back resulting in a different day */
-    tm_.tm_hour = 1;
+    /* Disabling DST: Set tm_isdst to -1 so that mktime() won't adjust for daylight saving time. */
+    tm_.tm_isdst = -1;
 
     mktime(&tm_); /* mktime modifies tm_ if it refers invalid date */
     if (tm.tm_mday != tm_.tm_mday) { /* e.g 2018-02-29 -> 2018-03-01 */
