@@ -318,7 +318,7 @@ test_invalid(void **state)
     struct lyd_node *diff = NULL;
 
     assert_int_equal(lyd_diff_siblings(model_1, lyd_child(model_1), 0, &diff), LY_EINVAL);
-    CHECK_LOG_CTX("Invalid arguments - cannot create diff for unrelated data (lyd_diff()).", NULL);
+    CHECK_LOG_CTX("Invalid arguments - cannot create diff for unrelated data (lyd_diff()).", NULL, 0);
 
     assert_int_equal(lyd_diff_siblings(NULL, NULL, 0, NULL), LY_EINVAL);
 
@@ -655,6 +655,18 @@ test_nested_list(void **state)
     xml1 =
             "<df xmlns=\"urn:libyang:tests:defaults\">"
             "  <list>"
+            "    <name>n0</name>"
+            "    <value>26</value>"
+            "    <list2>"
+            "      <name2>n22</name2>"
+            "      <value2>26</value2>"
+            "    </list2>"
+            "    <list2>"
+            "      <name2>n23</name2>"
+            "      <value2>26</value2>"
+            "    </list2>"
+            "  </list>"
+            "  <list>"
             "    <name>n1</name>"
             "    <value>25</value>"
             "    <list2>"
@@ -686,18 +698,6 @@ test_nested_list(void **state)
             "      <value2>26</value2>"
             "    </list2>"
             "  </list>"
-            "  <list>"
-            "    <name>n0</name>"
-            "    <value>26</value>"
-            "    <list2>"
-            "      <name2>n22</name2>"
-            "      <value2>26</value2>"
-            "    </list2>"
-            "    <list2>"
-            "      <name2>n23</name2>"
-            "      <value2>26</value2>"
-            "    </list2>"
-            "  </list>"
             "</df>";
     xml2 =
             "<df xmlns=\"urn:libyang:tests:defaults\">"
@@ -717,6 +717,14 @@ test_nested_list(void **state)
 
     CHECK_LYD_STRING(diff,
             "<df xmlns=\"urn:libyang:tests:defaults\" xmlns:yang=\"urn:ietf:params:xml:ns:yang:1\" yang:operation=\"none\">\n"
+            "  <list>\n"
+            "    <name>n0</name>\n"
+            "    <value yang:operation=\"replace\" yang:orig-default=\"false\" yang:orig-value=\"26\">30</value>\n"
+            "    <list2 yang:operation=\"delete\">\n"
+            "      <name2>n22</name2>\n"
+            "      <value2>26</value2>\n"
+            "    </list2>\n"
+            "  </list>\n"
             "  <list yang:operation=\"delete\">\n"
             "    <name>n1</name>\n"
             "    <value>25</value>\n"
@@ -749,14 +757,6 @@ test_nested_list(void **state)
             "      <value2>26</value2>\n"
             "    </list2>\n"
             "  </list>\n"
-            "  <list yang:operation=\"none\">\n"
-            "    <name>n0</name>\n"
-            "    <value yang:operation=\"replace\" yang:orig-default=\"false\" yang:orig-value=\"26\">30</value>\n"
-            "    <list2 yang:operation=\"delete\">\n"
-            "      <name2>n22</name2>\n"
-            "      <value2>26</value2>\n"
-            "    </list2>\n"
-            "  </list>\n"
             "</df>\n");
 
     lyd_free_all(data1);
@@ -976,13 +976,13 @@ test_userord_list(void **state)
             "    <l1>a</l1>\n"
             "    <l2 yang:operation=\"delete\">1</l2>\n"
             "  </ul>\n"
-            "  <ul yang:orig-key=\"[l1='a']\" yang:operation=\"replace\" yang:key=\"[l1='c']\">\n"
-            "    <l1>b</l1>\n"
-            "  </ul>\n"
             "  <ul yang:operation=\"none\">\n"
             "    <l1>c</l1>\n"
             "    <l2 yang:operation=\"replace\" yang:orig-default=\"false\" yang:orig-value=\"3\">33</l2>\n"
             "  </ul>\n"
+            "  <ul yang:orig-key=\"[l1='a']\" yang:operation=\"replace\" yang:key=\"[l1='c']\">\n"
+            "    <l1>b</l1>\n"
+            "  </ul>\n"
             "</df>\n";
 
     TEST_DIFF_3(xml1, xml2, xml3, out_diff_1, out_diff_2, out_merge);
