@@ -211,16 +211,22 @@ help(int shortout)
     printf("  -X, --extended-leafref\n"
             "                Allow usage of deref() XPath function within leafref\n\n");
 
-#ifndef NDEBUG
+    printf("  -J, --json-null\n"
+            "                Allow usage of JSON empty values ('null') within input data\n\n");
+
     printf("  -G GROUPS, --debug=GROUPS\n"
+#ifndef NDEBUG
             "                Enable printing of specific debugging message group\n"
             "                (nothing will be printed unless verbosity is set to debug):\n"
-            "                <group>[,<group>]* (dict, xpath, dep-sets)\n\n");
+            "                <group>[,<group>]* (dict, xpath, dep-sets)\n\n"
+#else
+            "                Unsupported for the Release build\n\n"
 #endif
+          );
 }
 
 static void
-libyang_verbclb(LY_LOG_LEVEL level, const char *msg, const char *path)
+libyang_verbclb(LY_LOG_LEVEL level, const char *msg, const char *data_path, const char *schema_path, uint64_t line)
 {
     char *levstr;
 
@@ -238,8 +244,12 @@ libyang_verbclb(LY_LOG_LEVEL level, const char *msg, const char *path)
         levstr = "dbg :";
         break;
     }
-    if (path) {
-        fprintf(stderr, "libyang %s %s (%s)\n", levstr, msg, path);
+    if (data_path) {
+        fprintf(stderr, "libyang %s %s (%s)\n", levstr, msg, data_path);
+    } else if (schema_path) {
+        fprintf(stderr, "libyang %s %s (%s)\n", levstr, msg, schema_path);
+    } else if (line) {
+        fprintf(stderr, "libyang %s %s (line %" PRIu64 ")\n", levstr, msg, line);
     } else {
         fprintf(stderr, "libyang %s %s\n", levstr, msg);
     }
@@ -381,7 +391,6 @@ fill_context_inputs(int argc, char *argv[], int optind, LYD_FORMAT data_in_forma
     return 0;
 }
 
-#ifndef NDEBUG
 /**
  * @brief Enable specific debugging messages.
  *
@@ -417,8 +426,6 @@ set_debug_groups(char *groups, struct yl_opt *yo)
     return 0;
 }
 
-#endif
-
 /**
  * @brief Process command line options and store the settings into the context.
  *
@@ -459,9 +466,8 @@ fill_context(int argc, char *argv[], struct yl_opt *yo, struct ly_ctx **ctx)
         {"yang-library",      no_argument,       NULL, 'y'},
         {"yang-library-file", required_argument, NULL, 'Y'},
         {"extended-leafref",  no_argument,       NULL, 'X'},
-#ifndef NDEBUG
-        {"debug",            required_argument, NULL, 'G'},
-#endif
+        {"json-null",         no_argument,       NULL, 'J'},
+        {"debug",             required_argument, NULL, 'G'},
         {NULL,               0,                 NULL, 0}
     };
     uint8_t data_type_set = 0;
@@ -472,12 +478,7 @@ fill_context(int argc, char *argv[], struct yl_opt *yo, struct ly_ctx **ctx)
     yo->line_length = 0;
 
     opterr = 0;
-#ifndef NDEBUG
-    while ((opt = getopt_long(argc, argv, "hvVQf:I:p:DF:iP:qs:neE:t:d:lL:o:O:R:myY:Xx:G:", options, &opt_index)) != -1)
-#else
-    while ((opt = getopt_long(argc, argv, "hvVQf:I:p:DF:iP:qs:neE:t:d:lL:o:O:R:myY:Xx:", options, &opt_index)) != -1)
-#endif
-    {
+    while ((opt = getopt_long(argc, argv, "hvVQf:I:p:DF:iP:qs:neE:t:d:lL:o:O:R:myY:XJx:G:", options, &opt_index)) != -1) {
         switch (opt) {
         case 'h': /* --help */
             help(0);
@@ -657,14 +658,16 @@ fill_context(int argc, char *argv[], struct yl_opt *yo, struct ly_ctx **ctx)
             yo->ctx_options |= LY_CTX_LEAFREF_EXTENDED;
             break;
 
-#ifndef NDEBUG
+        case 'J': /* --json-null */
+            yo->data_parse_options |= LYD_PARSE_JSON_NULL;
+            break;
+
         case 'G':   /* --debug */
             if (set_debug_groups(optarg, yo)) {
                 return -1;
             }
             break;
-            /* case 'G' */
-#endif
+
         default:
             YLMSG_E("Invalid option or missing argument: -%c.", optopt);
             return -1;
@@ -728,7 +731,7 @@ main_ni(int argc, char *argv[])
     uint32_t u;
 
     /* set callback for printing libyang messages */
-    ly_set_log_clb(libyang_verbclb, 1);
+    ly_set_log_clb(libyang_verbclb);
 
     r = fill_context(argc, argv, &yo, &ctx);
     if (r < 0) {
